;**************************************************************************************************
;
;						LCD.ASM				13-10-07
;
;						Outputs to 16x2 LCD in 4-bit mode
;
;**************************************************************************************************

;**************************************************************************************************

;	PROCESSOR mega 8  16
;	Clock = 16 MHz

;**************************************************************************************************

;**************************************************************************************************

;	PROCESSOR ATMEL
;
; 80-1 , C0-2 , 90-3 , D0-4 ,
;	  2  4  LCD   

;**************************************************************************************************

	.include "m8def.inc"


	.def Timer1	= r16	;   1ms
	.def TimerX	= r17	;   Xms
	.def Select	= r18	;  RS bit
	.def OutCod	= r19	;      
	.def Temp	= r21

	.EQU RS	= 6		;
	.EQU E	= 7		;
	.EQU OutPort =PortD
	.EQU OutStrob =PortB

;    LCD **********************************************************************

	.CSEG
	.ORG $0000			;  

		rjmp 	RESET	;reset entry vector

		
RESET:	ldi		temp, LOW(RAMEND) ;setup stack pointer
		out 	SPL, temp
		ldi		temp, HIGH(RAMEND)
		out		SPH, temp


		ldi		Temp,0xFF	
		out		DDRD,Temp
		out		DDRB,Temp

		rcall	Init
aa:		rcall	OutMes
		rjmp	aa


;        ******************************************************

; 80-1 , C0-2 , 90-3 , D0-4 ,

OutMes:	
		ldi		Select,(0<<RS)	; RS=0    
		ldi		Temp,0x85		; 8= 1 , 4= 5 
		rcall	Send			;   LCD
		ldi		Select,(1<<RS)	; RS=1   

		ldi		Temp,'N'		; 
		rcall	Send			;   LCD
		ldi		Temp,'O'				; 
		rcall	Send			;   LCD
		ldi		Temp,'K'				; 
		rcall	Send			;   LCD
		ldi		Temp,'I'				; 
		rcall	Send			;   LCD
		ldi		Temp,'A'
		rcall	Send			;   LCD
		
	

		ldi		Select,(0<<RS)	; RS=0    
		ldi		Temp,0xC4		; C= 2 , 4= 5 
		rcall	Send			;   LCD
		ldi		Select,(1<<RS)	; RS=1   


		ldi		Temp,'D'
		rcall	Send			;   LCD
		ldi		Temp,'r'
		rcall	Send			;   LCD
		ldi		Temp,'a'
		rcall	Send			;   LCD
		ldi		Temp,'i'				; 
		rcall	Send			;   LCD
		ldi		Temp,'v'				; 
		rcall	Send			;   LCD
		ldi		Temp,'e'
		rcall	Send			;   LCD
		ldi		Temp,'r'				; 
		rcall	Send			;   LCD
	
		ret			


;   4   RB4 - RB7 **************************************************************

Send:
		mov		OutCod,Temp		;  
		andi	Temp,0xF0		;   
		out		OutPort,Temp		;   
		sbrc	Select,RS		;  RS 
		sbi		OutStrob,RS		;   RS  
		rcall	PulseE			;   E
		rcall	Onems			;  1ms

		swap	OutCod			;     
		andi	OutCod,0xF0		;   
		out		OutPort,OutCod	;   
		sbrc	Select,RS		;  RS 
		sbi		OutStrob,RS		;   RS  
		rcall	PulseE			;   E
		rcall	Onems			;  1ms
		ret						; 
	
;   ....................................

	
Init:	ldi		TimerX,0x64		;   ,  100ms  64
		rcall	Xms				;  
		ldi		Select,0xF0		; Mask for select code
		ldi		Temp,0x30		; and output it to display
		out		OutPort,Temp
		rcall	PulseE			;   E
		ldi		TimerX,0x05		;   ,  5ms 
		rcall	Xms				;  
		rcall	PulseE			;   E
		rcall	Onems			;   1ms
		rcall	PulseE			;   E
		cbi		OutPort,4			;  4-bit 
		rcall	PulseE			;   E

		ldi		Temp,0x28		;  4-bit , 2 
		rcall	Send			; 
		ldi		Temp,0x08		; Switch off display
		rcall	Send			; 
		ldi		Temp,0x01		; Code to clear display
		rcall	Send			; 
		ldi		Temp,0x06		; Enable cursor auto inc  
		rcall	Send			; 
		ldi		Temp,0x80		; Zero display address
		rcall	Send			; 
		ldi		Temp,0x0C		; Turn on display  
		rcall	Send			; 
		ret


;  Xms, X   W ********************************************************************

Xms:							;  X ms
LoopX:	rcall	Onems			;  1ms
		dec		TimerX			;  X
		brne	LoopX			;   TimerX = 0
		ret						; 
	
;   1ms (1000 ) ********************************************************************
	
Onems:	ldi		Timer1,0xF9		;  249    = F9
Loop1:	nop						;    4 
		dec		Timer1			;   1 Timer1
		brne	Loop1			;   Timer1 = 0
		ret						; 

;    E ****************************************************************************

PulseE:	sbi		OutStrob,E			;  E
		rcall	Onems			;  1ms
		cbi		OutStrob,E			;  E
		rcall	Onems			;  1ms
		ret						; 

;**************************************************************************************************


	
end:
